pro correl_map, p0, p1, p2, p3,  xoffset=xoffset,  yoffset=yoffset, $
         apply=apply, _extra=_extra, smap0=smap0, smap1=smap1, $
         correl_option=correl_option, submap_only=submap_only
;+
;   Name: correl_map
;
;   Purpose: auto correlate overlapping images, return and optionally apply offests
;
;   Input Parameters:
;       map0,map1 - two map objects, for image0 and image1 respect.
;       -OR-
;       index0,data0,index1,data1 - two pair of ssw standard 'index,data' 
;
;   Output Parameters:
;      xoffset - X offset of image1 relative to image0 (data units)
;      yoffset - Y offset of image1 relative to image0 (data units)
;
;   Keyword Parameters:
;      apply - (input/switch) if set, apply x/y offsets to input (map1 or index1)
;      smap0 - (output) - sub map of overlapping pixel region from image0
;      smap1 - (output) - sub map of overlapping pixel regioi from image1
;      correl_option (input) - define correlation routine to use, including:
;                      1: correl_optimize.pro    
;      _extra - other keywords passed to 'correl_optimize.pro', including:
;          MAGNIFICATION = option to determine offsets up to fractional pixels,
;                          (example: MAG=2 means 1/2 pixel accuracy, default=1).
;         /NUMPIX: sqrt( sqrt( # pixels )) used as correlation weighting factor.
;         /MONITOR causes the progress of computation to be briefly printed.
;  
;   Method: uses DMZ mapping software for extraction, then apply auto-correlation
;           to define and optionally apply X/Y offsets.
;
;   TODO:
;      1. allow different auto-correlation function options (since there are
;         hundreds of auto-correlators out there in SSW...)
;      2. think about roll...
;
;   Side Effects:
;      if /APPLY set, input map1.xc/map1.yc -OR- index.XCEN/index.YCEN 
;      are updated to reflect auto-correlation offsets
; 
;   Restrictions:
;      CORREL_OPTION - only one option for today = 'correl_optimize.pro'
;
;   History:
;      10-Oct-2000 - S.L.Freeland
;-

if n_elements(correl_option) eq 0 then correl_option=1

apply=keyword_set(apply)
mapin=n_params() eq 2

case n_params() of 
   2: begin 
      if not valid_map(p0) and valid_map(p1) then begin 
         box_message,'Two parameter call - you must include two valud map objects'
         return
      endif
      map0=p0
      map1=p1
   endcase
   4: begin
         if data_chk(p0,/struct) and data_chk(p2,/struct) and $
            (data_chk(p1,/nimage) eq 1) and (data_chk(p3,/nimage) eq 1) then begin 
            index2map,p0,p1,map0          ; index,data->map object
            index2map,p2,p3,map1
         endif else begin 
            box_message,'Four parameter call - must include 2 pair of valid "index,data"'
            return
         endelse
   endcase
   else: begin 
      box_message,['IDL>correl_map, map0, map1 [,/apply] , xoff=xoff, yoff=yoff', $
                   '-OR-',$
                   'IDL>correl_map,index0,data0,index1,data1 [,/apply], xoff=xoff, yoff=yoff']
      return
   endcase
endcase

sub_map, map0, smap0, ref_map=map1       ; submap [map0 overlap with map1 ]
sub_map, map1, smap1, ref_map=smap0      ; submap [map1 overlap]

if keyword_set(submap_only) then begin
   box_message,'Just returning submaps via keywords SMAP0 and SMAP1'
   return
endif

case correl_option of 
   1: begin 
        correl_optimize, smap0.data, smap1.data, xpixoff, ypixoff, _extra=_extra
      endcase
   else: begin 
      box_message,'CORREL_OPTION ' + strtrim(correl_option,2) + ' not yet implemented'
   endcase
endcase

if required_tags(p1,'xc,yc,dx,dy') then begin
   xoffset=float(xpixoff)*p1.dx
   yoffset=float(ypixoff)*p1.dy
   if apply then begin 
      p1.xc=p1.xc+xoffset
      p1.yc=p1.yc+yoffset
   endif
endif else begin 
   if required_tags(p2,'xcen,ycen,cdelt1,cdelt2') then begin 
      xoffset=xpixoff*p2.cdelt1
      yoffset=ypixoff*p2.cdelt2
      if apply then begin 
         update_history,p2,/caller,'XOFFSET ' + string(xoffset,format='(f6.1)')
         update_history,p2,/caller,'YOFFSET ' + string(yoffset,format='(f6.1)')
         p2.xcen=p2.xcen+xoffset
         p2.ycen=p2.ycen+yoffset
      endif
   endif
endelse

if n_elements(xoffset) eq 0 then $
   box_message,'Non SSW standard '+ (['INDEX','MAP'])(mapin) + ' structures

return
end


